@@ -30,7 +30,11 @@ if ( !function_exists('dbg_error_log') ) {
     global $c, $session;
     $args = func_get_args();
     $type = "DBG";
-    $component = array_shift($args);
+    if (func_num_args() > 1) {
+      $component = array_shift($args);
+    } else {
+      $component = 'UNKNOWN';
+    }
     if ( substr( $component, 0, 3) == "LOG" ) {
       // Special escape case for stuff that always gets logged.
       $type = 'LOG';
@@ -50,18 +54,17 @@ if ( !function_exists('dbg_error_log') ) {
     /* filter by remote IP or logged-in user */
     if ( isset($c->dbg_filter["remoteIP"]) && !in_array($_SERVER['REMOTE_ADDR'], $c->dbg_filter["remoteIP"]) ) return;
     if ( isset($c->dbg_filter["authenticatedUser"]) ) {
-        if ( !isset($session->username) ) return;
-        if ( !in_array($session->username, $c->dbg_filter["authenticatedUser"]) ) return;
+      if ( !isset($session->username) ) return;
+      if ( !in_array($session->username, $c->dbg_filter["authenticatedUser"]) ) return;
     }
 
-    $argc = func_num_args();
-    if ( 2 <= $argc ) {
+    if ( count($args) >= 2 ) {
       $format = array_shift($args);
+      @error_log( $c->sysabbr.": $type: $component:". vsprintf( $format, $args ) );
     }
     else {
-      $format = "%s";
+      @error_log( $c->sysabbr.": $type: $component:" . $args[0]);
     }
-    @error_log( $c->sysabbr.": $type: $component:". vsprintf( $format, $args ) );
   }
 }
 
@@ -603,6 +606,7 @@ if ( !function_exists("force_utf8") ) {
 
 
 $timezone_identifiers_list_cache = timezone_identifiers_list();
+$timezone_identifiers_list_cache = isset($timezone_identifiers_list_cache) ? $timezone_identifiers_list_cache : [];
 
 /**
 * Try and extract something like "Pacific/Auckland" or "America/Indiana/Indianapolis" if possible.
@@ -700,6 +704,6 @@ if ( !function_exists("gzdecode") ) {
  */
 function awl_version() {
   global $c;
-$c->awl_library_version = 0.61;
+$c->awl_library_version = 0.64;
   return $c->awl_library_version;
 }
