@@ -142,7 +142,7 @@ class Session
   *
   * @param string $sid A session identifier.
   */
-  function Session( $sid="" )
+  function __construct( $sid="" )
   {
     global $sid, $sysname;
 
@@ -173,9 +173,9 @@ class Session
     else {
       $sql = "SELECT session.*, usr.* FROM session JOIN usr USING ( user_no )";
     }
-    $sql .= " WHERE session.session_id = ? AND (md5(session.session_start::text) = ? OR session.session_key = ?) ORDER BY session.session_start DESC LIMIT 2";
+    $sql .= " WHERE session.session_id = ? AND session.session_key = ? ORDER BY session.session_start DESC LIMIT 2";
 
-    $qry = new AwlQuery($sql, $session_id, $session_key, $session_key);
+    $qry = new AwlQuery($sql, $session_id, $session_key);
     if ( $qry->Exec('Session') && 1 == $qry->rows() ) {
       $this->AssignSessionDetails( $qry->Fetch() );
       $qry = new AwlQuery('UPDATE session SET session_end = current_timestamp WHERE session_id=?', $session_id);
@@ -198,10 +198,12 @@ class Session
   *
   * @param string $whatever A log string
   * @param mixed $whatever... Further parameters to be replaced into the log string a la printf
+  * @deprecated
   */
   function Log( $whatever )
   {
     global $c;
+    deprecated('Session::Log');
 
     $argc = func_num_args();
     $format = func_get_arg(0);
@@ -226,10 +228,12 @@ class Session
   * @param string $group The name of an arbitrary debug group.
   * @param string $whatever A log string
   * @param mixed $whatever... Further parameters to be replaced into the log string a la printf
+  * @deprecated
   */
   function Dbg( $whatever )
   {
     global $debuggroups, $c;
+    deprecated('Session::Dbg');
 
     $argc = func_num_args();
     $dgroup = func_get_arg(0);
@@ -363,14 +367,8 @@ class Session
             //  Create a cookie for the sesssion
             setcookie('sid',$sid, 0,'/');
             // Recognise that we have started a session now too...
-            $this->Session($sid);
+            $this->__construct($sid);
             dbg_error_log( "Login", " Login: New session $session_id started for $username ($user_no)" );
-            if ( isset($_POST['remember']) && intval($_POST['remember']) > 0 ) {
-              $cookie = md5( $user_no ) . ";";
-              $cookie .= session_salted_md5($user_no . $usr->username . $usr->password);
-              $GLOBALS['lsid'] = $cookie;
-              setcookie( "lsid", $cookie, time() + (86400 * 3600), "/" );   // will expire in ten or so years
-            }
             $this->just_logged_in = true;
 
             // Unset all of the submitted values, so we don't accidentally submit an unexpected form.
@@ -386,9 +384,9 @@ class Session
             else {
               $sql = "SELECT session.*, usr.* FROM session JOIN usr USING ( user_no )";
             }
-            $sql .= " WHERE session.session_id = ? AND (md5(session.session_start::text) = ? OR session.session_key = ?) ORDER BY session.session_start DESC LIMIT 2";
+            $sql .= " WHERE session.session_id = ? AND session.session_key = ? ORDER BY session.session_start DESC LIMIT 2";
 
-            $qry = new AwlQuery($sql, $session_id, $session_key, $session_key);
+            $qry = new AwlQuery($sql, $session_id, $session_key);
             if ( $qry->Exec('Session') && 1 == $qry->rows() ) {
               $this->AssignSessionDetails( $qry->Fetch() );
             }
@@ -427,100 +425,6 @@ class Session
 
 
 
-/**
-* Attempts to logs in using a long-term session ID
-*
-* This is all horribly insecure, but its hard not to be.
-*
-* @param string $lsid The user's value of the lsid cookie.
-* @return boolean Whether or not the user's lsid cookie got them in the door.
-*/
-  function LSIDLogin( $lsid ) {
-    global $c;
-    dbg_error_log( "Login", " LSIDLogin: Attempting login for $lsid" );
-
-    list($md5_user_no,$validation_string) = explode( ';', $lsid );
-    $qry = new AwlQuery( "SELECT * FROM usr WHERE md5(user_no::text)=? AND active", $md5_user_no );
-    if ( $qry->Exec('Login') && $qry->rows() == 1 ) {
-      $usr = $qry->Fetch();
-      list( $x, $salt, $y) = explode('*', $validation_string);
-      $my_validation = session_salted_md5($usr->user_no . $usr->username . $usr->password, $salt);
-      if ( $validation_string == $my_validation ) {
-        // Now get the next session ID to create one from...
-        $qry = new AwlQuery( "SELECT nextval('session_session_id_seq')" );
-        if ( $qry->Exec('Login') && $qry->rows() == 1 ) {
-          $seq = $qry->Fetch();
-          $session_id = $seq->nextval;
-          $session_key = md5( rand(1010101,1999999999) . microtime() );  // just some random shite
-          dbg_error_log( "Login", " LSIDLogin: Valid username/password for $username ($usr->user_no)" );
-
-          // And create a session
-          $sql = "INSERT INTO session (session_id, user_no, session_key) VALUES( ?, ?, ? )";
-          $qry = new AwlQuery( $sql, $session_id, $usr->user_no, $session_key );
-          if ( $qry->Exec('Login') ) {
-            // Assign our session ID variable
-            $sid = "$session_id;$session_key";
-
-            //  Create a cookie for the sesssion
-            setcookie('sid',$sid, 0,'/');
-            // Recognise that we have started a session now too...
-            $this->Session($sid);
-            dbg_error_log( "Login", " LSIDLogin: New session $session_id started for $this->username ($usr->user_no)" );
-
-            $this->just_logged_in = true;
-
-            // Unset all of the submitted values, so we don't accidentally submit an unexpected form.
-            unset($_POST['username']);
-            unset($_POST['password']);
-            unset($_POST['submit']);
-            unset($_GET['submit']);
-            unset($GLOBALS['submit']);
-
-            if ( function_exists('local_session_sql') ) {
-              $sql = local_session_sql();
-            }
-            else {
-              $sql = "SELECT session.*, usr.* FROM session JOIN usr USING ( user_no )";
-            }
-            $sql .= " WHERE session.session_id = ? AND (md5(session.session_start::text) = ? OR session.session_key = ?) ORDER BY session.session_start DESC LIMIT 2";
-
-            $qry = new AwlQuery($sql, $session_id, $session_key, $session_key);
-            if ( $qry->Exec('Session') && 1 == $qry->rows() ) {
-              $this->AssignSessionDetails( $qry->Fetch() );
-            }
-
-            $rc = true;
-            return $rc;
-          }
-   // else ...
-          $this->cause = 'ERR: Could not create new session.';
-        }
-        else {
-          $this->cause = 'ERR: Could not increment session sequence.';
-        }
-      }
-      else {
-        dbg_error_log( "Login", " LSIDLogin: $validation_string != $my_validation ($salt - $usr->user_no, $usr->username, $usr->password)");
-        $client_messages[] = i18n('Invalid username or password.');
-        if ( isset($c->dbg['Login']) || isset($c->dbg['ALL']) )
-          $this->cause = 'WARN: Invalid password.';
-        else
-          $this->cause = 'WARN: Invalid username or password.';
-      }
-    }
-    else {
-    $client_messages[] = i18n('Invalid username or password.');
-    if ( isset($c->dbg['Login']) || isset($c->dbg['ALL']) )
-      $this->cause = 'WARN: Invalid username.';
-    else
-      $this->cause = 'WARN: Invalid username or password.';
-    }
-
-    dbg_error_log( "Login", " LSIDLogin: $this->cause" );
-    return false;
-  }
-
-
 /**
 * Renders some HTML for a basic login panel
 *
@@ -531,7 +435,6 @@ class Session
     dbg_error_log( "Login", " RenderLoginPanel: action_target='%s'", $action_target );
     $userprompt = translate("User Name");
     $pwprompt = translate("Password");
-    $rememberprompt = str_replace( ' ', '&nbsp;', translate("forget me not"));
     $gobutton = htmlspecialchars(translate("GO!"));
     $gotitle = htmlspecialchars(translate("Enter your username and password then click here to log in."));
     $temppwprompt = translate("If you have forgotten your password then");
@@ -550,7 +453,6 @@ class Session
 <th class="prompt">$pwprompt:</th>
 <td class="entry">
 <input class="password" type="password" name="password" size="12" />
- &nbsp;<label>$rememberprompt: <input class="checkbox" type="checkbox" name="remember" value="1" /></label>
 </td>
 </tr>
 <tr>
@@ -730,7 +632,6 @@ EOTEXT;
     if ( ! $password_sent && ((isset($_POST['username']) && $_POST['username'] != "" )
                               || (isset($_POST['email_address']) && $_POST['email_address'] != "" )) ) {
       // Username or EMail were non-null, but we didn't find that user.
-
       $page_content = <<<EOTEXT
 <div id="logon">
 <h1>Unable to Reset Password</h1>
@@ -741,15 +642,14 @@ to arrange for an administrator to reset your password.</p>
 </div>
 EOTEXT;
     }
-
-    if ( $password_sent ) {
+    else if ( $password_sent ) {
       $page_content = <<<EOTEXT
 <div id="logon">
 <h1>Temporary Password Sent</h1>
 <p>A temporary password has been e-mailed to you.  This password
 will be valid for 24 hours and you will be required to change
 your password after logging in.</p>
-<p><a href="/">Click here to return to the login page.</a></p>
+<p><a href=".">Click here to return to the login page.</a></p>
 </div>
 EOTEXT;
     }
@@ -797,13 +697,20 @@ EOTEXT;
   static function _CheckLogout() {
     if ( isset($_GET['logout']) ) {
       dbg_error_log( "Login", ":_CheckLogout: Logging out");
+      if ( isset($_COOKIE['sid']) ) {
+        // clean up our session in the session table
+        list( $session_id, $session_key ) = explode( ';', $_COOKIE['sid'], 2 );
+        $sql = 'DELETE FROM session WHERE session_id = ? AND session_key = ?';
+        $qry = new AwlQuery( $sql, $session_id, $session_key );
+        $qry->Exec('Logout',__LINE__,__FILE__);
+        // expire sessions older than 16 hours
+        $sql = "DELETE FROM session WHERE session_end < current_timestamp - interval '16 hours'";
+        $qry = new AwlQuery( $sql );
+        $qry->Exec('Expire',__LINE__,__FILE__);
+      }
       setcookie( 'sid', '', 0,'/');
       unset($_COOKIE['sid']);
       unset($GLOBALS['sid']);
-      unset($_COOKIE['lsid']); // Allow a cookied person to be un-logged-in for one page view.
-      unset($GLOBALS['lsid']);
-
-      if ( isset($_GET['forget']) ) setcookie( 'lsid', '', 0,'/');
     }
   }
 
@@ -814,37 +721,25 @@ EOTEXT;
       $this->SendTemporaryPassword();
     }
     else if ( isset($_POST['username']) && isset($_POST['password']) ) {
+      $username = $_POST['username'];
       // Try and log in if we have a username and password
       $this->Login( $_POST['username'], $_POST['password'] );
-      @dbg_error_log( "Login", ":_CheckLogin: User %s(%s) - %s (%d) login status is %d", $_POST['username'], $this->fullname, $this->user_no, $this->logged_in );
-    }
-    else if ( !isset($_COOKIE['sid']) && isset($_COOKIE['lsid']) && $_COOKIE['lsid'] != "" ) {
-      // Validate long-term session details
-      $this->LSIDLogin( $_COOKIE['lsid'] );
-      dbg_error_log( "Login", ":_CheckLogin: User $this->username - $this->fullname ($this->user_no) login status is $this->logged_in" );
+      @dbg_error_log( "Login", ":_CheckLogin: User %s - %s (%d) login status is %d", $username, $this->fullname, $this->user_no, $this->logged_in );
     }
     else if ( !isset($_COOKIE['sid']) && isset($c->authenticate_hook['server_auth_type']) ) {
       /**
-      * The authentication should have happened in the server, and we should accept it if so.
+      * The authentication has happened in the server, and we should accept it if so.
       */
-      if ( is_array($c->authenticate_hook['server_auth_type']) ) {
-        if ( in_array( strtolower($_SERVER['AUTH_TYPE']), array_map('strtolower', $c->authenticate_hook['server_auth_type']) )) {
-          if (isset($_SERVER["REMOTE_USER"]))
-            $this->Login($_SERVER['REMOTE_USER'], "", true);  // Password will not be checked.
-          else
-            $this->Login($_SERVER['REDIRECT_REMOTE_USER'], "", true);  // Password will not be checked.
-        }
-      }
-      else if ( strtolower($c->authenticate_hook['server_auth_type']) == strtolower($_SERVER['AUTH_TYPE']) ) {
-        /**
-        * Perhaps this 'split' is not a good idea though.  People may want to use the
-        * full ID as the username.  A further option may be desirable.
-        */
+      if ( ( is_array($c->authenticate_hook['server_auth_type'])
+             && in_array( strtolower($_SERVER['AUTH_TYPE']), array_map('strtolower', $c->authenticate_hook['server_auth_type'])) )
+      ||
+           ( !is_array($c->authenticate_hook['server_auth_type'])
+             && strtolower($c->authenticate_hook['server_auth_type']) == strtolower($_SERVER['AUTH_TYPE']) )
+      ) {
         if (isset($_SERVER["REMOTE_USER"]))
-          list($username) = explode('@', $_SERVER['REMOTE_USER']);
+          $this->Login($_SERVER['REMOTE_USER'], "", true);  // Password will not be checked.
         else
-          list($username) = explode('@', $_SERVER['REDIRECT_REMOTE_USER']);
-        $this->Login($username, "", true);  // Password will not be checked.
+          $this->Login($_SERVER['REDIRECT_REMOTE_USER'], "", true);  // Password will not be checked.
       }
     }
   }
@@ -938,7 +833,7 @@ EOTEXT;
       dbg_error_log('Session',':CheckConfirmationHash: Salt "%s"', $salt );
       $test_against = session_salted_md5( $this->session_start.$varname.$this->session_key, $salt ) ;
       dbg_error_log('Session',':CheckConfirmationHash: Testing against "%s"', $test_against );
-      
+
       return ($hashwegot == $test_against);
     }
     return false;
