@@ -40,6 +40,13 @@ class XMLDocument {
   */
   private $root;
 
+  /**
+  * This appears to not be used, but was being dynamically added.
+  * Setting to public in case it is used somewhere, for something.
+  * @var next_prefix
+  */
+  public $next_prefix;
+
   /**
   * Simple XMLDocument constructor
   *
@@ -170,12 +177,12 @@ class XMLDocument {
   static public $ns_caldav = 'urn:ietf:params:xml:ns:caldav';
   static public $ns_carddav = 'urn:ietf:params:xml:ns:carddav';
   static public $ns_calendarserver = 'http://calendarserver.org/ns/';
-  
+
   /**
   * Special helper for namespaced tags.
   *
-  * @param object $element The tag are adding a new namespaced element to
-  * @param string $tag the tag name, possibly prefixed with the namespace
+  * @param object $element The tag we are adding a new namespaced element to
+  * @param string $tag The tag name, possibly prefixed with the namespace
   * @param mixed  $content The content of the tag
   * @param array  $attributes An array of key/value pairs of attributes.
   * @param string $namespace The namespace for the tag
@@ -204,8 +211,8 @@ class XMLDocument {
   /**
   * Special helper for tags in the DAV: namespace.
   *
-  * @param object $element The tag are adding a new namespaced element to
-  * @param string $tag the tag name
+  * @param object $element The tag we are adding a new namespaced element to
+  * @param string $tag The tag name
   * @param mixed  $content The content of the tag
   * @param array  $attributes An array of key/value pairs of attributes.
   */
@@ -217,8 +224,8 @@ class XMLDocument {
   /**
   * Special helper for tags in the urn:ietf:params:xml:ns:caldav namespace.
   *
-  * @param object $element The tag are adding a new namespaced element to
-  * @param string $tag the tag name
+  * @param object $element The tag we are adding a new namespaced element to
+  * @param string $tag The tag name
   * @param mixed  $content The content of the tag
   * @param array  $attributes An array of key/value pairs of attributes.
   */
@@ -231,8 +238,8 @@ class XMLDocument {
   /**
   * Special helper for tags in the urn:ietf:params:xml:ns:carddav namespace.
   *
-  * @param object $element The tag are adding a new namespaced element to
-  * @param string $tag the tag name
+  * @param object $element The tag we are adding a new namespaced element to
+  * @param string $tag The tag name
   * @param mixed  $content The content of the tag
   * @param array  $attributes An array of key/value pairs of attributes.
   */
@@ -243,10 +250,10 @@ class XMLDocument {
 
 
   /**
-  * Special helper for tags in the urn:ietf:params:xml:ns:caldav namespace.
+  * Special helper for tags in the http://calendarserver.org/ns/ namespace.
   *
-  * @param object $element The tag are adding a new namespaced element to
-  * @param string $tag the tag name
+  * @param object $element The tag we are adding a new namespaced element to
+  * @param string $tag The tag name
   * @param mixed  $content The content of the tag
   * @param array  $attributes An array of key/value pairs of attributes.
   */
