@@ -43,6 +43,7 @@ class XMLElement {
       $this->content = $content;
     }
     $this->attributes = $attributes;
+    if ( $this->attributes === false ) $this->attributes = array();
     if ( isset($xmlns) ) {
       $this->xmlns = $xmlns;
     }
@@ -259,7 +260,17 @@ class XMLElement {
       * Render the content, with special characters escaped
       *
       */
-      $r .= htmlspecialchars($this->content, ENT_NOQUOTES );
+      if(strpos($this->content, '<![CDATA[')===0 && strrpos($this->content, ']]>')===strlen($this->content)-3)
+        $r .= '<![CDATA[' . str_replace(']]>', ']]]]><![CDATA[>', substr($this->content, 9, -3)) . ']]>';
+      else if ( defined('ENT_XML1') && defined('ENT_DISALLOWED') )
+        // Newer PHP versions allow specifying ENT_XML1, but default to ENT_HTML401.  Go figure.  #PHPWTF
+        $r .= htmlspecialchars($this->content, ENT_NOQUOTES |  ENT_XML1 | ENT_DISALLOWED );
+      // Need to work out exactly how to do this in PHP.
+      // else if ( preg_match('{^[\t\n\r\x0020-\xD7FF\xE000-\xFFFD\x10000-\x10FFFF]+$}u', utf8ToUnicode($this->content)) )
+      //   $r .= '<![CDATA[' . $this->content . ']]>';
+      else
+        // Older PHP versions default to ENT_XML1.
+        $r .= htmlspecialchars($this->content, ENT_NOQUOTES );
     }
     return $r;
   }
@@ -306,10 +317,10 @@ class XMLElement {
 //      printf( "4: %s: %s\n", $this->tagname, $this->xmlns );
       $attr .= sprintf( ' xmlns="%s"', $this->xmlns);
     }
-    
+
     $r .= substr("                        ",0,$indent) . '<' . $tagname . $attr;
 
-    if ( (is_array($this->content) && count($this->content) > 0) || (!is_array($this->content) && strlen($this->content) > 0) ) {
+    if ( isset($this->content) && ((is_array($this->content) && count($this->content) > 0) || (!is_array($this->content) && strlen($this->content) > 0)) ) {
       $r .= ">";
       $r .= $this->RenderContent($indent,$nslist,$force_xmlns);
       $r .= '</' . $tagname.">\n";
@@ -335,22 +346,26 @@ class XMLElement {
 * @param intref &$start_from A pointer to our current integer offset into $xmltags
 * @return mixed Either a single XMLElement, or an array of XMLElement objects.
 */
-function BuildXMLTree( $xmltags, &$start_from ) {
+function BuildXMLTree( $xmltags, &$start_from = 0 ) {
   $content = array();
 
-  if ( !isset($start_from) ) $start_from = 0;
-
   for( $i=0; $i < 50000 && isset($xmltags[$start_from]); $i++) {
     $tagdata = $xmltags[$start_from++];
+
     if ( !isset($tagdata) || !isset($tagdata['tag']) || !isset($tagdata['type']) ) break;
+
     if ( $tagdata['type'] == "close" ) break;
+
     $xmlns = null;
     $tag = $tagdata['tag'];
+
     if ( preg_match( '{^(.*):([^:]*)$}', $tag, $matches) ) {
       $xmlns = $matches[1];
       $tag = $matches[2];
     }
+
     $attributes = ( isset($tagdata['attributes']) ? $tagdata['attributes'] : false );
+
     if ( $tagdata['type'] == "open" ) {
       $subtree = BuildXMLTree( $xmltags, $start_from );
       $content[] = new XMLElement($tag, $subtree, $attributes, $xmlns );
