@@ -206,7 +206,7 @@ class Editor
       else if ( is_string($fields) ) {
         // We've been given a table name, so get all fields for it.
         $this->BaseTable = $fields;
-        $field_list = get_fields($fields);
+        $field_list = awl_get_fields($fields);
         foreach( $field_list AS $k => $v ) {
           $this->AddField($k);
         }
@@ -540,7 +540,7 @@ class Editor
     // $matches[0] is the complete match
     switch( $matches[0] ) {
       case "##form##": /** @todo It might be nice to construct a form ID */
-        return sprintf('<form method="POST" enctype="multipart/form-data" class="editor" id="%s">', $this->Id);
+        return sprintf('<form method="POST" enctype="multipart/form-data" class="editor" id="form_%s">', $this->Id);
       case "##submit##":
         $action =  ( $this->RecordAvailable ? 'update' : 'insert' );
         $submittype = ($this->RecordAvailable ? translate('Apply Changes') : translate('Create'));
