@@ -53,12 +53,10 @@ class vProperty extends vObject {
     /**#@-*/
 
     /**
-     * The constructor parses the incoming string, which is formatted as per RFC2445 as a
-     *   propname[;param1=pval1[; ... ]]:propvalue
-     * however we allow ourselves to assume that the RFC2445 content unescaping has already
-     * happened when myComponent::ParseFrom() called myComponent::UnwrapComponent().
+     * Parsing of the incoming string is now performed lazily, in ParseFromIterator.
+     * You should use getter methods such as Value() and getParameterValue() instead of direct access
+     * to $content, $parameters etc, to ensure that parsing has occurred.
      *
-     * @param HeapLines $heapLines The string from the myComponent which contains this property.
      */
     function __construct( $name = null, &$master = null, &$refData = null, $seek = null ) {
         parent::__construct($master);
@@ -73,11 +71,8 @@ class vProperty extends vObject {
         unset($this->content);
         unset($this->parameters);
 
-
-
         if ( isset($refData)){
 
-
             if(gettype($refData) == 'object') {
                 $this->iterator = &$refData;
                 $this->seek = &$seek;
@@ -85,39 +80,43 @@ class vProperty extends vObject {
             } else {
                 $this->line = $refData;
 
-                /*  test */
-                //$this->ParseFrom($refData);
-                //unset($this->line);
-
-
                 unset($this->iterator);
                 unset($this->seek);
             }
-            //$this->ParseFrom($refData);
-            //unset($this->iterator);
-            //unset($this->seek);
         } else {
             unset($this->iterator);
             unset($this->seek);
 
         }
-
     }
 
-
     /**
-     * The constructor parses the incoming string, which is formatted as per RFC2445 as a
+     * Parses the incoming string, which is formatted as per RFC2445 as a
      *   propname[;param1=pval1[; ... ]]:propvalue
-     * however we allow ourselves to assume that the RFC2445 content unescaping has already
+     * However we allow ourselves to assume that the RFC2445 content unescaping has already
      * happened when myComponent::ParseFrom() called myComponent::UnwrapComponent().
      *
-     * @param string $propstring The string from the myComponent which contains this property.
+     * Note this function is called lazily, from the individual getter methods. This avoids the cost of parsing at
+     * the point of object instantiation.
      */
-    function ParseFrom( &$unescaped ) {
-        //$this->rendered = $heapLines;
-        // temporady like string
-        //$this->rendered = (strlen($propstring) < 73 ? $propstring : null);  // Only pre-rendered if we didn't unescape it
+    function ParseFromIterator()
+    {
+        $unescaped;
 
+        if (isset($this->iterator)) {
+            $this->iterator->seek($this->seek);
+            $unescaped = $this->iterator->current();
+        } else if (isset($this->line)) {
+            $unescaped = $this->line;
+        } else {
+            $unescaped = '';
+        }
+
+        $this->ParseFrom($unescaped);
+        unset($unescaped);
+    }
+
+    function ParseFrom( &$unescaped ) {
         // unescape \r and \n in the value
         $unescaped = preg_replace( array('{\\\\[nN]}', '{\\\\[rR]}'), array("\n", "\r"), $unescaped);
 
@@ -179,7 +178,7 @@ class vProperty extends vObject {
                     $name = strtoupper($v);
                     $value = null;
                 }
-                if ( preg_match( '{^"(.*)"$}', $value, $matches) ) {
+                if ( isset($value) && preg_match( '{^"(.*)"$}', $value, $matches) ) {
                     $value = $matches[1];
                 }
                 if ( isset($this->parameters[$name]) && is_array($this->parameters[$name]) ) {
@@ -196,25 +195,6 @@ class vProperty extends vObject {
     }
 
 
-    function ParseFromIterator(){
-        $unescaped;
-
-        if(isset($this->iterator)){
-            $this->iterator->seek($this->seek);
-            $unescaped = $this->iterator->current();
-
-
-        } else if(isset($this->line)){
-          $unescaped = $this->line;
-        } else {
-            $unescaped = '';
-        }
-
-        $this->ParseFrom($unescaped);
-        unset($unescaped);
-
-    }
-
     /**
      * Get/Set name property
      *
@@ -327,7 +307,7 @@ class vProperty extends vObject {
 //    dbg_error_log('PUT', $this->name.$this->RenderParameters().':'.$this->content );
     }
 
-    /**                                                                
+    /**
      * Clear all parameters, or the parameters matching a particular type
      * @param string|array $type The type of parameters or an
      * array associating parameter names with true values: array( 'PARAMETER' => true, 'PARAMETER2' => true )
@@ -402,6 +382,7 @@ class vProperty extends vObject {
 
         $property = preg_replace( '/[;].*$/', '', $this->name );
         $escaped = $this->content;
+        $property = preg_replace( '/^.*[.]/', '', $property ); //temporarily remove grouping prefix from CARDDAV attributes ("item1.", "item2.", etc)
         switch( $property ) {
             /** Content escaping does not apply to these properties culled from RFC2445 */
             case 'ATTACH':                case 'GEO':                       case 'PERCENT-COMPLETE':      case 'PRIORITY':
@@ -411,16 +392,16 @@ class vProperty extends vObject {
             case 'RRULE':                 case 'REPEAT':                    case 'TRIGGER':               case 'RDATE':
             case 'COMPLETED':             case 'DTEND':                     case 'DUE':                   case 'DTSTART':
             case 'DTSTAMP':               case 'LAST-MODIFIED':             case 'CREATED':               case 'EXDATE':
+            case 'CATEGORIES' :
             break;
 
             /** Content escaping does not apply to these properties culled from RFC6350 / RFC2426 */
-            case 'ADR':                case 'N':
+            case 'ADR':                case 'N':            case 'ORG':
             // escaping for ';' for these fields also needs to happen to the components they are built from.
             $escaped = str_replace( '\\', '\\\\', $escaped);
             $escaped = preg_replace( '/\r?\n/', '\\n', $escaped);
             $escaped = str_replace( ',', '\\,', $escaped);
             break;
-
             /** Content escaping applies by default to other properties */
             default:
                 $escaped = str_replace( '\\', '\\\\', $escaped);
@@ -429,7 +410,6 @@ class vProperty extends vObject {
         }
 
         $rendered = '';
-
         $property = sprintf( "%s%s:", $this->name, $this->RenderParameters() );
         if ( (strlen($property) + strlen($escaped)) <= 72 ) {
             $rendered = $property . $escaped;
@@ -453,13 +433,14 @@ class vProperty extends vObject {
     /**
      * Test a PROP-FILTER or PARAM-FILTER and return a true/false
      * PROP-FILTER (is-defined | is-not-defined | ((time-range | text-match)?, param-filter*))
-     * PARAM-FILTER (is-defined | is-not-defined | ((time-range | text-match)?, param-filter*))
+     * PARAM-FILTER (is-not-defined | text-match?)
      *
      * @param array $filter An array of XMLElement defining the filter
+     * @param bool  $matchAll If true, all given filters must match (AND), otherwise a single match suffices (OR)
      *
      * @return boolean Whether or not this vProperty passes the test
      */
-    function TestFilter( $filters ) {
+    function TestFilter( $filters, $matchAll = true ) {
         foreach( $filters AS $k => $v ) {
             $tag = $v->GetNSTag();
 //      dbg_error_log( 'vCalendar', "vProperty:TestFilter: '%s'='%s' => '%s'", $this->name, $tag, $this->content );
@@ -480,64 +461,37 @@ class vProperty extends vObject {
 
                 case 'urn:ietf:params:xml:ns:carddav:text-match':
                 case 'urn:ietf:params:xml:ns:caldav:text-match':
-                    $search = $v->GetContent();
-                    $haystack = $this->content;
-                    $match = isset($haystack);
-                    if ( $match ) {
-                             $collation = $v->GetAttribute("collation");
-                             switch( strtolower($collation) ) {
-                             case 'i;octet':
-                                 // don't change search and haystack
-                                 break;
-                             case 'i;ascii-casemap':
-                             case 'i;unicode-casemap':
-                             default:
-                                 // for ignore case search we transform
-                                 // search and haystack to lowercase
-                                 $search   = strtolower( $search );
-                                 $haystack = strtolower( $haystack );
-                                 break;
-                             }
-
-                             $matchType = $v->GetAttribute("match-type");
-                             switch( strtolower($matchType) ) {
-                             case 'equals':
-                                 $match = ( $haystack === $search );
-                                 break;
-                             case 'starts-with':
-                                $length = strlen($search);
-                                 if ($length == 0) {
-                                     $match = true;
-                                 } else {
-                                     $match = !strncmp($haystack, $search, $length);
-                                 }
-                                 break;
-                             case 'ends-with':
-                                $length = strlen($search);
-                                 if ($length == 0) {
-                                     $match = true;
-                                 } else {
-                                     $match = ( substr($haystack, -$length) === $search );
-                                 }
-                                 break;
-                             default: // contains
-                                 $match = strstr( $haystack, $search );
-                                 break;
-                             }
-                    }
-
-                    $negate = $v->GetAttribute("negate-condition");
-                    if ( isset($negate) && strtolower($negate) == "yes" ) {
-                             $match = !$match;
-                    }
-                    if ( ! $match ) return false;
+                    // Call the Value() getter method to get hold of the vProperty content - need to ensure parsing has occurred
+                    $haystack = $this->Value();
+                    $match = $this->TestTextMatch($v, $haystack);
+                    // Early exit: OR match can stop if we have on success, AND match can stop if we have a fail
+                    if ( $match != $matchAll ) return $match;
                     break;
 
                 case 'urn:ietf:params:xml:ns:carddav:param-filter':
                 case 'urn:ietf:params:xml:ns:caldav:param-filter':
                     $subfilter = $v->GetContent();
                     $parameter = $this->GetParameterValue($v->GetAttribute("name"));
-                    if ( ! $this->TestParamFilter($subfilter,$parameter) ) return false;
+
+                    if (empty($subfilter)) {
+                        // param-filter without subfilter matches if the parameter is defined
+                        $match = isset($parameter);
+                    } else {
+                        if (!isset($parameter)) {
+                            $parameter = [ null ];
+                        } elseif (!is_array($parameter)) {
+                            $parameter = explode(',', $parameter);
+                        }
+
+                        $match = false;
+                        foreach ($parameter as $param) {
+                            $match = $this->TestParamFilter($subfilter,$param);
+                            if ($match) break;
+                        }
+                    }
+
+                    // Early exit: OR match can stop if we have on success, AND match can stop if we have a fail
+                    if ( $match != $matchAll ) return $match;
                     break;
 
                 default:
@@ -545,23 +499,87 @@ class vProperty extends vObject {
                     break;
             }
         }
-        return true;
+
+        // AND-case: all matched; OR-case: all failed
+        return $matchAll;
     }
 
     function fill($sp, $en, $pe){
 
     }
 
+    function TestTextMatch( $filter, $haystack ) {
+        // if the property / parameter is not defined, the prop-filter/param-filter fails
+        if (!isset($haystack)) {
+            return false;
+        }
+
+        $search = $filter->GetContent();
+        $collation = $filter->GetAttribute("collation");
+        if (! isset($collation)) {
+            $collation = '';
+        }
+        switch( strtolower($collation) ) {
+            case 'i;octet':
+                // don't change search and haystack
+                break;
+            case 'i;ascii-casemap':
+            case 'i;unicode-casemap':
+            default:
+                // for ignore case search we transform
+                // search and haystack to lowercase
+                $search   = strtolower( $search );
+                $haystack = strtolower( $haystack );
+                break;
+        }
+
+        $matchType = $filter->GetAttribute("match-type");
+        if (! isset($matchType)) {
+            $matchType = '';
+        }
+        switch( strtolower($matchType) ) {
+            case 'equals':
+                $match = ( $haystack === $search );
+                break;
+            case 'starts-with':
+                $length = strlen($search);
+                if ($length == 0) {
+                    $match = true;
+                } else {
+                    $match = !strncmp($haystack, $search, $length);
+                }
+                break;
+            case 'ends-with':
+                $length = strlen($search);
+                if ($length == 0) {
+                    $match = true;
+                } else {
+                    $match = ( substr($haystack, -$length) === $search );
+                }
+                break;
+            default: // contains
+                $length = strlen($search);
+                if ($length == 0) {
+                    $match = true;
+                } else {
+                    $match = strstr( $haystack, $search );
+                }
+                break;
+        }
+
+        $negate = $filter->GetAttribute("negate-condition");
+        if ( isset($negate) && strtolower($negate) == "yes" ) {
+            $match = !$match;
+        }
+
+        return $match;
+    }
+
     function TestParamFilter( $filters, $parameter_value ) {
         foreach( $filters AS $k => $v ) {
             $subtag = $v->GetNSTag();
 //      dbg_error_log( 'vCalendar', "vProperty:TestParamFilter: '%s'='%s' => '%s'", $this->name, $subtag, $parameter_value );
             switch( $subtag ) {
-                case 'urn:ietf:params:xml:ns:caldav:is-defined':
-                case 'urn:ietf:params:xml:ns:carddav:is-defined':
-                    if ( empty($parameter_value) ) return false;
-                    break;
-
                 case 'urn:ietf:params:xml:ns:caldav:is-not-defined':
                 case 'urn:ietf:params:xml:ns:carddav:is-not-defined':
                     if ( ! empty($parameter_value) ) return false;
@@ -573,13 +591,7 @@ class vProperty extends vObject {
 
                 case 'urn:ietf:params:xml:ns:carddav:text-match':
                 case 'urn:ietf:params:xml:ns:caldav:text-match':
-                    $search = $v->GetContent();
-                    $match = false;
-                    if ( !empty($parameter_value) ) $match = strstr( $this->content, $search );
-                    $negate = $v->GetAttribute("negate-condition");
-                    if ( isset($negate) && strtolower($negate) == "yes" ) {
-                        $match = !$match;
-                    }
+                    $match = $this->TestTextMatch($v, $parameter_value);
                     if ( ! $match ) return false;
                     break;
 
